angular.module('myApp')

// SCORE
// 20 bombs, 42 probes, 5:49
// 20 bombs, 41 probes, 4:10
// 20 bombs, 46 probes, 3:30

.factory('mainService', function() {
    var main = {};

    main.allclear = false;

    main.groundSize = 500;
    main.bombCount = 20;

    main.bombSize = 8; //WARNING value harcoded for display

    main.nodes = [];
    main.bombs = [];
    main.flags = [];

    main.counters = {
        usedProbes : 0
    };

    main.nodeModels = [
        {type:'probe', items:[], size:8, area:80},
        {type:'flag', area:5},
    ]

    var lastProbeID;

    var addHandlers = {};

    function distance(p1, p2) {
        return Math.sqrt(Math.pow(p1.x-p2.x, 2) + Math.pow(p1.y-p2.y, 2));
    }

    addHandlers['probe'] = function(probe) {
/*        for (var b in main.bombs) {
            var dist = distance(main.bombs[b].pos, node.pos);
            if (dist <= node.area) {
                node.items.push({dist:dist})
            }
        }*/
        probe.id = ++lastProbeID;
        main.counters.usedProbes++;
        var explosion = detectItems(probe);
        if (explosion) {
            return true;
        }
        main.nodes.push(probe);
        return false;
    }

    addHandlers['flag'] = function(node) {
        main.flags.push(node);
        flagNearBombs(node);
        return false;
    }

    // Sonar
    function detectItems(probe) {
        var explosion = false;
        probe.items.splice(0,probe.items.length);
        main.bombs.filter(function(bomb){return !bomb.disabled}).forEach(function(bomb) {
            var dist = distance(bomb.pos, probe.pos);
            if (dist <= probe.area) {
                probe.items.push({dist:dist})
                if (dist <= probe.size) {
                    bomb.exploded = true;
                    explosion = true;
                }
            }
        })
        !probe.items.length && (probe.cleared = true);
        return explosion;
    }

    function flagNearBombs(flag, remove) {
        main.bombs.forEach(function(bomb) {
            var dist = distance(bomb.pos, flag.pos);
            if (dist <= flag.area) {
                bomb.flagged += remove ? -1 : 1;
                bomb.flag = remove ? null : flag;
            }
        })
    }

    main.addNode = function(node) {
        return addHandlers[node.type](node);
    }

    main.removeFlag = function(flag) {
        var idx = main.flags.findIndex(function(f) {return f == flag});
        main.flags.splice(idx,1);
        flagNearBombs(flag, true);
    }

    main.removeProbe = function(probe) {
        var idx = main.nodes.findIndex(function(p) {return p == probe});
        main.nodes.splice(idx,1);
    }

    // Check number of flags == number of bombs in the area
    main.canClear = function(probe) {
        var flagsInArea = main.flags.filter(function(f) {
            return !f.used && (distance(probe.pos, f.pos) <= probe.area+3);
        });
        return flagsInArea.length >= probe.items.length;
    }

    main.clearArea = function(probe) {
        var explosion = false;
        // Disable all flagged bombs in the probe area
        // If not flagged bomb in area then explode => game over
        for (var b in main.bombs) {
            var bomb = main.bombs[b];
            var dist = distance(bomb.pos, probe.pos);
            if (dist <= probe.area) {
                if (bomb.flagged) {
                    bomb.disabled = true;
                    bomb.flag.used = true;
                } else {
                    bomb.exploded = true;
                    explosion = true;
                }
            }
        }
        if (!explosion) {
            probe.cleared = true;
            // Update all probes
            main.nodes.forEach(detectItems);
            // Check if all bombs are disabled
            main.allclear = (main.bombCount == main.bombs.filter(function(b) {return b.disabled}).length);
        }
        return explosion;
    }


    function collide(pos) {
        var idx = main.bombs.findIndex(function(b) {
            return distance(b.pos,pos) <= 2*main.bombSize;
        })
        return (idx >= 0);
    }

    main.init = function() {
        main.nodes.splice(0,main.nodes.length);
        main.bombs.splice(0,main.bombs.length);
        main.flags.splice(0,main.flags.length);

        lastProbeID = 0;
        main.counters.usedProbes = 0;

        for (var n=0;n<main.bombCount;n++) {
            var pos;
            do {
                pos = {
                    x : Math.random() * main.groundSize,
                    y : Math.random() * main.groundSize,
                }
            } while (collide(pos))
            main.bombs.push({
                id : n,
                pos : pos,
                flagged : 0,
                disabled : false,
                exploded : false,
            })
        }
    }

    return main;
})
;